import { createReactAgent } from '@langchain/langgraph/prebuilt';
import { MemorySaver } from '@langchain/langgraph';
import { ChatOllama } from '@langchain/ollama';
import { ChatOpenAI } from '@langchain/openai';
import { client } from './mongo.ts';
import { MongoDBSaver } from '@langchain/langgraph-checkpoint-mongodb';
import { nanoid } from 'nanoid';
import { HumanMessage } from '@langchain/core/messages';
export { HumanMessage };

export const agentModelList = ['qwen2.5:14b', 'qwen2.5-coder:7b', 'llama3.1:8b', 'bakllava:latest', 'gpt-4o'] as const;
export type AiAgentModel = (typeof agentModelList)[number];
export type AiAgentCache = 'memory' | 'mongodb';
export type AiAgentOpts = {
  id: string;
  type: 'ollama' | 'openai';
  model: AiAgentModel;
  baseUrl: string;
  apiKey?: string;
  temperature?: number;
  cache?: AiAgentCache;
  cacheName?: string;
};
export type AiAgentStatus = 'ready' | 'loading' | 'error';
// export const CreateAgent = (opts: CreateAgentOptions) => {
//   const;
// };

export class AiAgent {
  agent: ReturnType<typeof createReactAgent>;
  agentModel: ChatOllama | ChatOpenAI;
  memorySaver: MemorySaver | MongoDBSaver;
  id: string;
  baseUrl: string;
  type: 'ollama' | 'openai';
  model: AiAgentModel;
  apiKey: string;
  temperature = 0;
  cache?: AiAgentCache;
  cacheName?: string;
  status?: 'ready' | 'loading' | 'error';
  constructor(opts?: AiAgentOpts) {
    this.type = opts?.type || 'ollama';
    this.baseUrl = opts?.baseUrl || 'http://localhost:11434';
    this.model = opts?.model;
    this.apiKey = opts?.apiKey;
    this.temperature = opts?.temperature || 0;
    this.cache = opts?.cache || 'mongodb';
    this.cacheName = opts?.cacheName || 'checkpointer';
    this.id = opts?.id || nanoid(8);
    if (this.type === 'openai') {
      if (!this.apiKey) {
        throw new Error('apiKey is required for openai agent');
      }
    }
    this.status = 'loading';
    this.createAgent();
  }
  createAgent() {
    this.createAgentModel();
    this.createMemoerSaver();
    if (this.status === 'error') {
      return;
    }

    const agentModel = this.agentModel;
    const memoerSaver = this.memorySaver;
    this.agent = createReactAgent({
      llm: agentModel,
      tools: [],
      checkpointSaver: memoerSaver,
    });
    this.status = 'ready';
  }
  createAgentModel() {
    const type = this.type;
    const model = this.model;
    const temperature = this.temperature;
    const apiKey = this.apiKey;
    const baseUrl = this.baseUrl;
    let agentModel;
    try {
      if (type === 'ollama') {
        agentModel = new ChatOllama({ temperature, model, baseUrl });
      } else if (type === 'openai') {
        agentModel = new ChatOpenAI(
          { temperature, model, apiKey },
          {
            baseURL: baseUrl,
          },
        );
      }
    } catch (e) {
      console.error('loading model error', e);
      this.status = 'error';
      return;
    }
    this.agentModel = agentModel;
    return this;
  }
  createMemoerSaver() {
    const cache = this.cache;
    const cacheName = this.cacheName;
    let memorySaver;
    try {
      if (cache === 'memory') {
        memorySaver = new MemorySaver();
      } else if (cache === 'mongodb') {
        memorySaver = new MongoDBSaver({ client, dbName: cacheName });
      }
    } catch (e) {
      console.error(e);
      this.status = 'error';
      return;
    }
    this.memorySaver = memorySaver;
  }
  sendHumanMessage(message: string, opts?: { thread_id: string }) {
    const mesage = new HumanMessage(message);
    return this.agent.invoke({ messages: [mesage] }, { configurable: { thread_id: 'test_human', ...opts } });
  }
  close() {
    // 清除 memory saver
    this.memorySaver = null;
    this.agentModel = null;
    this.agent = null;
  }
  async testQuery() {
    const id = this.id;
    try {
      const agent = this.agent;
      const message = new HumanMessage('你好');
      const res = await agent.invoke({ messages: [message] }, { configurable: { thread_id: 'test_ping' } });
      if (res) {
        return res;
      }
    } catch (e) {
      console.error(`test query [${id}]:`, e);
      this.status = 'error';
    }
  }
}
